﻿Module Module_Utils

    ' =======================================================================================================
    '   APP TITLE AND VERSION (ALL FIELDS)
    ' =======================================================================================================
    Friend Function AppTitleAndVersion(Optional ByVal RemoveThereminoPrefix As Boolean = False) As String
        Dim appname As String = IO.Path.GetFileNameWithoutExtension(Application.ExecutablePath)
        Dim Title As String = Replace(appname, "_", " ")
        If RemoveThereminoPrefix Then Title = Replace(Title, "Theremino ", "")
        Return Title + " - V" + GetVersionString()
    End Function

    ' =======================================================================================================
    '   GET VERSION (ALSO TRIPLE OR QUADRUPLE FIELDS)
    ' =======================================================================================================
    Friend Function GetVersionString() As String
        Dim s() As String = Split(My.Application.Info.Version.ToString, ".")
        GetVersionString = s(0) + "." + s(1)
        If s(2) <> "0" Then
            GetVersionString += "." + s(2)
            If s(3) <> "0" Then
                GetVersionString += "." + s(3)
            End If
        End If
    End Function

    ' =======================================================================================================
    '   GET BUFFERS AND IMAGES PATHS
    ' =======================================================================================================
    Friend Function GetBuffersPath() As String
        Dim s As String = IO.Path.GetFileNameWithoutExtension(Application.ExecutablePath)
        s = Replace(s, "Theremino_SignalScope", "")
        Return Application.StartupPath + "\Buffers" + s + "\"
    End Function

    Friend Function GetImagesPath() As String
        Dim s As String = IO.Path.GetFileNameWithoutExtension(Application.ExecutablePath)
        s = Replace(s, "Theremino_SignalScope", "")
        Return Application.StartupPath + "\Images" + s + "\"
    End Function

    ' =======================================================================================================
    '   CULTURE INFO FOR STRING CONVERSIONS
    ' =======================================================================================================
    Friend GCI As Globalization.CultureInfo = Globalization.CultureInfo.InvariantCulture

    ' =======================================================================================================
    '   MIXED FUNCTIONS
    ' =======================================================================================================
    Friend Function StringToInt32(ByVal s As String) As Int32
        Dim n As Double = Val(s)
        If n < Int32.MinValue Or n > Int32.MaxValue Then Return 0
        Return CInt(n)
    End Function


    'Friend Function ReplaceMultipleSpacesAndTrim(ByVal s As String) As String
    '    s = s.Replace(vbTab, " ")
    '    While s.Contains("  ")
    '        s = s.Replace("  ", " ")
    '    End While
    '    Return s.Trim
    'End Function
    'Friend Function RemoveComments(ByVal s As String) As String
    '    Dim i As Int32 = s.IndexOf("'")
    '    If i > 0 Then s = s.Remove(i).TrimEnd
    '    If i = 0 Then s = ""
    '    Return s
    'End Function



    '' =========================================================================
    ''  WINDOWS SCHEDULER PRECISION
    '' ========================================================================= 
    'Private Declare Function timeBeginPeriod Lib "winmm.dll" (ByVal uPeriod As Int32) As Int32
    'Private Declare Function timeEndPeriod Lib "winmm.dll" (ByVal uPeriod As Int32) As Int32
    'Friend Sub SchedulerMaxPrecision()
    '    If OperatingSystemIsWindows Then
    '        timeBeginPeriod(1)
    '    End If
    'End Sub
    'Friend Sub SchedulerDefaultPrecision()
    '    If OperatingSystemIsWindows Then
    '        timeEndPeriod(1)
    '    End If
    'End Sub

    Friend Sub InitPictureboxImage(ByVal pbox As PictureBox)
        With pbox
            If .ClientSize.Width < 1 Or .ClientSize.Height < 1 Then Return
            .Image = New Bitmap(.ClientSize.Width, .ClientSize.Height)
        End With
    End Sub

    'Friend Sub InitBitmap(ByRef bmp As Bitmap, ByVal w As Int32, ByVal h As Int32)
    '    If w < 1 Or h < 1 Then Return
    '    bmp = New Bitmap(w, h)
    'End Sub

    Friend Sub Combo_SetIndex(ByVal combo As ComboBox, ByVal index As Int32)
        If combo.Items.Count < 1 Then Exit Sub
        If index < 0 Then index = 0
        If index > combo.Items.Count - 1 Then index = combo.Items.Count - 1
        combo.SelectedIndex = index
    End Sub


    'Friend Function SecondsToMinSec(ByVal seconds As Double) As String
    '    Dim t As TimeSpan = TimeSpan.FromSeconds(seconds)
    '    Return t.Minutes.ToString("00") + ":" + t.Seconds.ToString("00")
    'End Function

    Friend Function RemoveComments(ByVal s As String) As String
        Dim i As Int32 = s.IndexOf("'")
        If i > 0 Then s = s.Remove(i).TrimEnd
        If i = 0 Then s = ""
        Return s
    End Function

    ' ========================================================
    '   PROCESS START AND WAIT
    ' ========================================================
    Friend Sub ProcessStartAndWait(ByVal fname As String)
        If Not My.Computer.FileSystem.FileExists(fname) Then Return
        Dim p As Process = Process.Start(fname)
        Do
            System.Threading.Thread.Sleep(100)
        Loop Until p.HasExited
    End Sub

    ' ========================================================
    '   SWAP
    ' ========================================================
    Public Sub Swap(Of T)(ByRef a As T, ByRef b As T)
        Dim temp As T
        temp = a
        a = b
        b = temp
    End Sub


    ' =============================================================
    '  LOAD BUFFER
    ' =============================================================
    Friend Sub LoadBuffer()
        Dim ofd As OpenFileDialog = New OpenFileDialog()
        ofd.InitialDirectory = GetBuffersPath()
        IO.Directory.CreateDirectory(ofd.InitialDirectory)
        ofd.Filter = "Buffer file (*.csv)|*.csv"
        ofd.DefaultExt = ".csv"
        ofd.Multiselect = True
        If ofd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            LoadBufferFrom(ofd.FileName)
        End If
    End Sub

    Friend Sub LoadBufferFrom(ByVal filename As String)
        If My.Computer.FileSystem.FileExists(filename) Then
            ' --------------------------------------------------------- set oldRunState
            Dim oldRunState As Boolean = Form1.btn_Run.Checked
            Form1.btn_Run.Checked = False
            StartStopAcquisition(False)
            ' -------------------------------------------------------
            fifo1.Flush()
            Dim buf() As FIFO.FifoEntry = fifo1.GetBuffer
            Dim f As System.IO.StreamReader
            f = IO.File.OpenText(filename)
            Dim i As Int32
            ' -------------------------------------------------------
            For i = 0 To buf.Length - 1
                If f.EndOfStream Then Exit For
                Dim p() As String = f.ReadLine.Split(","c)
                With buf(i)
                    If p.Length = 4 Then
                        .Value1 = CSng(Val(p(0)))
                        .Value2 = CSng(Val(p(1)))
                        .Value3 = CSng(Val(p(2)))
                        .Value4 = CSng(Val(p(3)))
                    ElseIf p.Length = 2 Then
                        .Value1 = CSng(Val(p(0)))
                        .Value2 = CSng(Val(p(1)))
                    End If
                End With
            Next
            fifo1.SetWriteIndex(i)
            fifo1.SetNumEntries(i)
            f.Close()
            ' ------------------------------------------------------- restore old run state
            If oldRunState Then
                Form1.btn_Run.Checked = True
                StartStopAcquisition(True)
            End If
        End If
    End Sub

    ' =============================================================
    '  SAVE BUFFER
    ' =============================================================
    Friend Sub SaveBuffer()
        If Form1.btn_Cursors.Checked Then
            If MessageBox.Show("Warning: Cursors are enabled." + vbCrLf + _
                               "Only the area between the two cursors will be saved." + vbCrLf + vbCrLf + _
                               "OK to proceed and save?", _
                               "Message from Theremino SignalScope", _
                               MessageBoxButtons.OKCancel, _
                               MessageBoxIcon.Information) <> Windows.Forms.DialogResult.OK Then Return
        End If
        Dim sfd As SaveFileDialog = New SaveFileDialog()
        sfd.InitialDirectory = GetBuffersPath()
        IO.Directory.CreateDirectory(sfd.InitialDirectory)
        sfd.DefaultExt = ".csv"
        sfd.Filter = "Buffer file (*.csv)|*.csv"
        sfd.FileName = "Buffer_" & Date.Now.ToString("yyyy_MM_dd_HH_mm_ss")
        If sfd.ShowDialog() = Windows.Forms.DialogResult.OK Then
            SaveBufferTo(sfd.FileName)
        End If
    End Sub

    Friend Sub SaveBufferAuto()
        Dim s As String = GetBuffersPath()
        IO.Directory.CreateDirectory(s)
        s += "Buffer_" + Date.Now.ToString("yyyy_MM_dd_HH_mm_ss") + ".csv"
        SaveBufferTo(s)
    End Sub

    Friend Sub SaveBufferTo(ByVal filename As String, Optional ByVal SaveAllBuffer As Boolean = False)
        Dim Buffer() As FIFO.FifoEntry = fifo1.GetBuffer
        IO.Directory.CreateDirectory(IO.Path.GetDirectoryName(filename))
        Dim FormatString As String = "+0.000000E+0;-0.000000E+0"
        Dim index As Int32
        Using sw As IO.StreamWriter = New IO.StreamWriter(filename)
            ' -------------------------------------------------------
            If Form1.btn_Cursors.Checked And Not SaveAllBuffer Then
                ' --------------------------------------------------- get cursor indexes and eventually swap
                Dim c1 As Int32 = GetCursor1_FifoIndex()
                Dim c2 As Int32 = GetCursor2_FifoIndex()
                If c1 > c2 Then Swap(c1, c2)
                ' --------------------------------------------------- save the cursor area
                For i As Int32 = c1 To c2
                    index = i
                    index = fifo1.CorrectIndex(index)
                    With Buffer(index)
                        sw.WriteLine(.Value1.ToString(FormatString, GCI) + ", " + _
                                     .Value2.ToString(FormatString, GCI) + ", " + _
                                     .Value3.ToString(FormatString, GCI) + ", " + _
                                     .Value4.ToString(FormatString, GCI))
                    End With
                Next
            Else
                ' --------------------------------------------------- save all the buffer
                Dim writeindex As Int32 = fifo1.GetWriteIndex
                Dim len As Int32 = fifo1.GetNumEntries
                If BufferLenMS \ 2 < len Then len = BufferLenMS \ 2
                For i As Int32 = writeindex - len To writeindex - 1
                    index = i
                    index = fifo1.CorrectIndex(index)
                    With Buffer(index)
                        sw.WriteLine(.Value1.ToString(FormatString, GCI) + ", " + _
                                     .Value2.ToString(FormatString, GCI) + ", " + _
                                     .Value3.ToString(FormatString, GCI) + ", " + _
                                     .Value4.ToString(FormatString, GCI))
                    End With
                Next
            End If
        End Using
    End Sub


    ' =============================================================================================
    '   DateTime to Singles / Singles to DateTime
    ' =============================================================================================
    '
    ' DateTime.Tick = 100 nS 
    ' 1 millisec = 10 000 Tick
    '
    ' One Year = 32 536 000 seconds
    ' One Year = 32 536 000 000 milliseconds
    ' One Year = 325 360 000 000 000 Ticks
    '
    ' Int64 = more than 280 000 000 years in milliseconds
    ' Int64 = more than 28 000 years in Ticks
    '
    ' ------------------------------------------------------------------------- Convert DateTime to 2 Singles
    Friend Function DateTimeToSingles(ByVal dt As DateTime) As Single()
        Dim i As Int64 = dt.Ticks
        Dim b() As Byte = BitConverter.GetBytes(i)
        Dim s(1) As Single
        s(0) = BitConverter.ToSingle(b, 0)
        s(1) = BitConverter.ToSingle(b, 4)
        Return s
    End Function
    '
    ' ------------------------------------------------------------------------- Convert 2 Singles to DateTime
    Friend Function SinglesToDateTime(ByVal s1 As Single, ByVal s2 As Single) As DateTime
        Dim b1() As Byte = BitConverter.GetBytes(s1)
        Dim b2() As Byte = BitConverter.GetBytes(s2)
        Dim b(7) As Byte
        b(0) = b1(0)
        b(1) = b1(1)
        b(2) = b1(2)
        b(3) = b1(3)
        b(4) = b2(0)
        b(5) = b2(1)
        b(6) = b2(2)
        b(7) = b2(3)
        Dim i As Int64 = BitConverter.ToInt64(b, 0)
        If i < 0 Or i > DateTime.MaxValue.Ticks Then
            Return New DateTime(0)
        End If
        Return New DateTime(i)
    End Function


    ' ==============================================================
    '  Test SlotFile and reload it if changed
    ' ============================================================== 
    Friend Sub TestSlotFile()
        ' -------------------------------------------------- reduce the test frequency if called by a fast timer
        Static counter As Int32
        counter += 1
        If counter < 3 Then Return ' 3 = 300 mS (with a 10 Hz Timer)
        counter = 0
        ' --------------------------------------------------  
        Static SlotFileInfo As IO.FileInfo = Nothing
        Dim newinfo As IO.FileInfo = My.Computer.FileSystem.GetFileInfo(SlotFilePathAndName)
        If SlotFileInfo Is Nothing Then
            SlotFileInfo = newinfo
        End If
        If newinfo.LastWriteTime <> SlotFileInfo.LastWriteTime Then
            SlotFileInfo = newinfo
            LoadSlotNames()
            Form1.SetSlotParams()
        End If
    End Sub

End Module
